import cv2
from glob import glob
import os
import subprocess


class Data(object):
    def __init__(self, src, transform=None):
        if isinstance(src, str):
            src = sorted(glob(src))
        self.src = src
        self.transform = transform

    def __len__(self):
        return len(self.src)

    def __getitem__(self, index):
        if isinstance(index, slice):
            return Data(self.src[index], self.transform)
        if self.transform is None:
            return self.src[index]
        return self.transform(self.src[index])

    def __iadd__(self, other):
        self.src += other.src
        return self

    def __add__(self, other):
        return Data(self.src + other.src, self.transform)


def imwrite(p, image):
    cv2.imwrite(p, image)


def imread(p):
    return cv2.imread(p, cv2.IMREAD_ANYCOLOR | cv2.IMREAD_ANYDEPTH)


def imread_mono(p):
    return cv2.imread(p, cv2.IMREAD_GRAYSCALE | cv2.IMREAD_ANYDEPTH)


def run_command(cmd):
    return subprocess.run(cmd, check=True)


def wget(url, path):
    run_command(['wget', '-c', '-O', path, url])


def unzip(path, dir):
    run_command(['unzip', '-u', path, '-d', dir])


def path(*children):
    return os.path.join(data_dir, *children)


def create_data_dir():
    os.makedirs(data_dir, exist_ok=True)


data_dir = os.path.join(os.path.realpath(os.path.dirname(__file__)), 'data')
create_data_dir()
