from argparse import ArgumentParser
from data import Data, imread, imread_mono, path, unzip, wget
import intrinsics


def download():
    for f in ('calibration_sequence_I.zip', 'Smooth_surfaces.zip'):
        wget('http://adas.cvc.uab.es/projects/multimodal/datasets/dataset/%s' % f, path(f))
        unzip(path(f), path())


def calibration_images():
    res = {
        # Color images
        # Sony ICX084 Bayer pattern CCD, 6mm focal length lens, 640×480 image resolution (Bumblebee 2 stereo).
        # 43-deg HFOV
        # https://www.ptgrey.com/bumblebee2-03-mp-color-firewire-1394a-6mm-sony-icx424-camera
        'left': Data(path('calibration_sequence_I', '*_BB_Left.ppm'), imread),
        'right': Data(path('calibration_sequence_I', '*_BB_Right.ppm'), imread),
        # IR images
        # PathFindIR Flir camera, 19mm focal length lens (36 degree HFOV), 534×426 image resolution.
        # We take negative image (with offset) to match the "polarity" of color images.
        'ir': Data(path('calibration_sequence_I', '*_IR1_crop.bmp'), lambda p: 255 - imread_mono(p))
    }
    return res


def images():
    res = {
        'left': (Data(path('Smooth_surfaces', 'Validation', '*_VS_L.ppm'), imread) +
                 Data(path('Smooth_surfaces', 'Car_I', '*_BB_Left.ppm'), imread)),
        'right': (Data(path('Smooth_surfaces', 'Validation', '*_VS_R.ppm'), imread) +
                  Data(path('Smooth_surfaces', 'Car_I', '*_BB_Right.ppm'), imread)),
        'ir': (Data(path('Smooth_surfaces', 'Validation', '*_IR_Cropped.bmp'), imread_mono) +
               Data(path('Smooth_surfaces', 'Car_I', '*_IR1_crop.ppm'), imread_mono))
    }
    return res


def calibrate_intrinsics():
    for camera, images in calibration_images().items():
        camera_matrix, distortion_coeffs = intrinsics.calibrate(images, checkerboard_pattern, checkerboard_points)
        calib_path = path('multimodal_%s_intrinsics.json' % camera)
        intrinsics.save(calib_path, camera_matrix, distortion_coeffs)
        print('Intrinsics saved to %s.' % calib_path)


def main():
    parser = ArgumentParser()
    parser.add_argument('action', choices=['download', 'calibrate'])
    args = parser.parse_args()

    if args.action == 'download':
        download()
    elif args.action == 'calibrate':
        calibrate_intrinsics()


cameras = ('left', 'right', 'ir')
checkerboard_pattern = (8, 9)
checkerboard_unit = (0.061, 0.047, 0.0)
checkerboard_points = intrinsics.checkerboard_points(checkerboard_pattern, checkerboard_unit)

if __name__ == '__main__':
    main()
